/******************************************************************************* 
 * Copyright (c) 2017, Real-Time Technology Solutions, Inc. All Rights Reserved. 
 *******************************************************************************/
package com.rttsweb.querysurge.rest;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.UriBuilder;

import org.glassfish.jersey.client.ClientConfig;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.rttsweb.querysurge.util.FileUtil;

// http://www.vogella.com/tutorials/REST/article.html
// https://stackoverflow.com/questions/16788213/jackson-how-to-transform-jsonnode-to-arraynode-without-casting

public class JerseyRestHandler {

	/**
	 * @param args
	 * 
	 * http://services.groupkt.com/country/search?text=un
	 * @throws URISyntaxException 
	 * @throws IOException 
	 * @throws JsonMappingException 
	 * @throws JsonParseException 
	 */
	public static final String HTTPPROTOCOL = "http://";
	
	private String mediaType = null;
	private String jsonResultArray = null;
	private String [] jsonArrayElementNames = null;
	private Client restClient = null;
	
	public JerseyRestHandler(String jsonResultArray, String [] jsonArrayElementNames) {
		ClientConfig jerseyClientConfig = new ClientConfig();
        this.restClient = ClientBuilder.newClient(jerseyClientConfig);
        this.mediaType = MediaType.APPLICATION_JSON;
        this.jsonResultArray = jsonResultArray;
        this.jsonArrayElementNames = jsonArrayElementNames;
	}
	
	public JerseyRestHandler(Client restClient, String mediaType, String jsonResultArray, String [] jsonArrayElementNames) {
		this.restClient = restClient;
		this.mediaType = mediaType;
		this.jsonResultArray = jsonResultArray;
		this.jsonArrayElementNames = jsonArrayElementNames;
	}
	
	public String getRestResponse(String baseURL, String urlPath) throws URISyntaxException {
		return getRestResponse(baseURL, urlPath, null, null);
	}
	
	public String getRestResponse(String baseURL, String urlPath, String [] searchKeys, String [] vals) throws URISyntaxException {
        URI uri = new URI(HTTPPROTOCOL + baseURL);
        UriBuilder builder = UriBuilder.fromUri(uri).path(urlPath);
        
        if ((searchKeys != null) && (vals != null) && (searchKeys.length > 0) && (vals.length > 0)) {
        	for (int n = 0; n < vals.length; n++) {
        		builder.queryParam(searchKeys[n], vals[n]);
        	}
        }
        
        URI baseUri = builder.build();
        WebTarget target = restClient.target(baseUri);
        
        String jsonResponse = target.request().accept(mediaType).get(String.class);
        return jsonResponse;
	}
	
	public void jsonToCsv(String jsonContent, String outputFilePath) throws JsonParseException, JsonMappingException, IOException {
		String csv = jsonToCsvFormat(jsonContent);
		FileUtil fut = new FileUtil();
		fut.writeFile(outputFilePath, csv);
	}
	
	public String jsonToCsvFormat(String jsonContent) throws JsonParseException, JsonMappingException, IOException {
		StringBuilder sb = new StringBuilder();
		ObjectMapper mapper = new ObjectMapper();
		JsonNode rootNode = mapper.readTree(jsonContent);
		
		List<JsonNode> resultNodeList = rootNode.findValues(jsonResultArray);
		JsonNode resultNode = resultNodeList.get(0);
		
		if (resultNode.isArray()) {
		    for (final JsonNode elemNode : resultNode) {
		    	boolean cycleEnd = false;
		    	for (int n = 0; n < jsonArrayElementNames.length; n++) {
		    		if (n == jsonArrayElementNames.length - 1) {
		    			cycleEnd = true;
		    		}
		    		String elemName = jsonArrayElementNames[n];
		    		List<JsonNode> jsonArrayElem = elemNode.findValues(elemName);
		    		checkJsonNodeListWithAppend(sb, jsonArrayElem, cycleEnd);
		    	}    
		    }
		}
		return sb.toString();
	}
	
	public void checkJsonNodeListWithAppend(StringBuilder sb, List<JsonNode> jn, boolean isLineEnd) {
		checkJsonNodeListWithAppend(sb, jn, 0, ",", isLineEnd);
	}
	
	public void checkJsonNodeListWithAppend(StringBuilder sb, List<JsonNode> jn, int listItem, String delim, boolean isLineEnd) {
		final String LINETERM = "\n";
		if ((jn != null) && (jn.size() > 0)  && (!jn.get(listItem).isNull()) && (!jn.get(listItem).isMissingNode())) {
    		sb.append(jn.get(listItem).asText());
    		if (!isLineEnd) {
    			sb.append(delim);
    		} else {
    			sb.append(LINETERM);
    		}
    	} else {
    		if (!isLineEnd) {
    			sb.append(delim);
    		} else {
    			sb.append(LINETERM);
    		}
    	}
	}
}
